classdef meanProjection
    % holds information about an individual mean (across epochs) projection from (multiple) dipole space to head (brain) grid
    % locations.
    properties
        headGrid;
        similarity;
        standardDeviationOfEstimatedDipoleLocation = 12; % in mm, default std. for gaussian cloud representing each dipole.
        convergence = [];
        convergenceSignificance = [];
        domain = [];% array containing domains (areas with similar projections).
        numberOfPermutations = 2000;
        linearizedProjectedMeasure;
        dipoleDensity;
        domainNumberCube = [];
        significanceLevelForDomain
        numberOfStandardDeviationsToTruncatedGaussaian = 3;
    end % properties
    
    methods(Static)
        function [projectionMatrix totalDipoleDenisty gaussianWeightMatrix]= getProjectionMatrix(dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfStandardDeviationsToTruncatedGaussaian, regionOfInterestCube, normalizeInBrainDipoleDenisty)
            % gaussianWeightMatrix is dipoles x (requested) grid points. It contains dipole denisty
            % at each grid point for each dipole.
                                              
            if nargin < 5 || isempty(regionOfInterestCube)
                regionOfInterestCube = headGrid.insideBrainCube;
            end;
            
            if nargin < 6
                normalizeInBrainDipoleDenisty = true; % by default normalize in-brain dipole density.
            end;
            
            if ischar(regionOfInterestCube) && strcmpi(regionOfInterestCube, 'all')
                regionOfInterestCube = true(headGrid.cubeSize);
            end;
            
            standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo = standardDeviationOfEstimatedDipoleLocation ^ 2;
            
            % projection matrix is number of dipoles x number of grid point inside brain volume
            numberOfPointsInTheResgionOfInterest = sum(regionOfInterestCube(:));
            
            projectionMatrix = zeros(size(dipole.location , 1), numberOfPointsInTheResgionOfInterest);
            totalDipoleDenisty = zeros(1, numberOfPointsInTheResgionOfInterest);
            gaussianWeightMatrix = zeros(size(dipole.location , 1), numberOfPointsInTheResgionOfInterest);
            distanceFromDipoleToGridLocationMatrix = zeros(size(dipole.location , 1), numberOfPointsInTheResgionOfInterest);
            
            
            % a N x 3 matrix (N is the number of grid points inside brain volume
            gridPosition = [headGrid.xCube(regionOfInterestCube) headGrid.yCube(regionOfInterestCube) headGrid.zCube(regionOfInterestCube);];
            
            if normalizeInBrainDipoleDenisty
                dipoleInBrainDensityNormalizationFactor = pr.meanProjection.calculateDipoleInBrainDensityNormalizationFactor(dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfStandardDeviationsToTruncatedGaussaian);
            end;
            
            for dipoleNumber = 1:size(dipole.location , 1)
                % first place distance in the array
                distanceFromDipoleToGridLocationMatrix(dipoleNumber,:) = sum( (gridPosition - repmat(dipole.location(dipoleNumber,:), size(gridPosition,1), 1)) .^2,2 ) .^ 0.5;
                gaussianWeightMatrix(dipoleNumber,:) = sqrt(1/(2 * pi * standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo)) * exp(-distanceFromDipoleToGridLocationMatrix(dipoleNumber,:).^2 / (2 * standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo));
                
                % truncate the dipole denisty Gaussian at ~3 standard deviation
                gaussianWeightMatrix(dipoleNumber, distanceFromDipoleToGridLocationMatrix(dipoleNumber,:) > (numberOfStandardDeviationsToTruncatedGaussaian * standardDeviationOfEstimatedDipoleLocation) ) = 0;
                
                % normalize the dipole in-brain denisty (make it sum up to one)
                if normalizeInBrainDipoleDenisty
                    gaussianWeightMatrix(dipoleNumber,:) = gaussianWeightMatrix(dipoleNumber,:) * dipoleInBrainDensityNormalizationFactor(dipoleNumber);
                end;
            end;
            
            % normalize gaussian weights to have the sum of 1 at each grid location
            for gridId = 1:size(gaussianWeightMatrix, 2)
                totalDipoleDenisty(gridId) = sum(gaussianWeightMatrix(:, gridId));
                if totalDipoleDenisty(gridId) > 0
                    projectionMatrix(:, gridId) = gaussianWeightMatrix(:, gridId) / totalDipoleDenisty(gridId);
                end;
            end;
            
        end
        function dipoleInBrainDensityNormalizationFactor = calculateDipoleInBrainDensityNormalizationFactor(dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfStandardDeviationsToTruncatedGaussaian)
            % calculate the factors (scalar values, each for a dipole) that normalize the dipole
            % projected density inside brain volume (makes its sum to be equal to one).
            
            % get each dipole density from gaussianWeightMatrix
            [projectionMatrix totalDipoleDenisty gaussianWeightMatrix]= pr.meanProjection.getProjectionMatrix(dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfStandardDeviationsToTruncatedGaussaian, headGrid.insideBrainCube, false);
            
            % calculate its sum inside brain volume, 1/this gives the factor which should be
            % multiplied by dipole denisty in projection (because we assume dipole should be
            % somewhere in the brain volume).
            dipoleInBrainDensityNormalizationFactor = repmat(1, size(gaussianWeightMatrix,1),1) ./ sum(gaussianWeightMatrix, 2);
        end;
        function [bestValue meanConvergence ord]  = findBestStandardDeviationOfEstimatedDipoleLocation(dipoleAndMeasure, headGrid, valuesToTry, plotResult) % static function, does not need an instance of the object to run.
            
            if nargin < 4
                plotResult = false;
            end;
            
            for i = 1:length(valuesToTry)
                [convergenceSignificance convergence] = calculate_measure_convergence_significance(dipoleAndMeasure.getPairwiseCorrelationSimilarity, dipoleAndMeasure.location, headGrid, valuesToTry(i), 0);
                meanConvergence(i) = mean(convergence(:));
            end;
            
            [maxMeanConvergence ord] = max(meanConvergence);
            bestValue = valuesToTry(ord);
            
            if plotResult
                figure;
                plot(valuesToTry, meanConvergence,'bo-', 'linewidth',2);
                hold on;
                plot(bestValue, maxMeanConvergence, 'sb', 'linewidth',5); % show max value in the plot with a filled rectangle.
            end;
        end
        function [sumWeightedSimilarityPvalue sumWeightedSimilarity] = calculateMeasureConvergenceSignificance(similarity, dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfPermutations, numberOfStandardDeviationsToTruncatedGaussaian, normalizeInBrainDipoleDenisty)
            
            if nargin<4
                standardDeviationOfEstimatedDipoleLocation = 12; % mm
            end;
            
            if nargin < 5
                numberOfPermutations = 2000;
            end;
            
            if nargin < 6
                % number of standard deviations after which the Gaussian function is truncated
                numberOfStandardDeviationsToTruncatedGaussaian = 3;
            end;
            
            if nargin < 7
                normalizeInBrainDipoleDenisty = true;
            end;
            
            sumWeightedSimilarity = headGrid.xCube * 0;
            
            standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo = standardDeviationOfEstimatedDipoleLocation ^ 2;
            
            similarity = similarity - diag(diag(similarity)); % remove ones on the diagonal
            dipoleLocation = dipole.location;
            
            surrogateSumWeightedSimilarity = zeros(1, numberOfPermutations);
            sumWeightedSimilarityPvalue = ones(size(headGrid.xCube));
            
            clear randomPermutation;
            for permutationNumber = 1:numberOfPermutations
                % randomPermutation(permutationNumber,:) = randperm(size(similarity,1)); % without repetition
                randomPermutation(permutationNumber,:) = randi(size(similarity,1), 1, size(similarity,1)); % with repetition
            end;
            
            pr.progress('init'); % start the text based progress bar
            
            if normalizeInBrainDipoleDenisty
                dipoleInBrainDensityNormalizationFactor = pr.meanProjection.calculateDipoleInBrainDensityNormalizationFactor(dipole, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfStandardDeviationsToTruncatedGaussaian);
            end;
            
            for i=1:numel(headGrid.xCube)
                if headGrid.insideBrainCube(i)
                    
                    if mod(i,10) ==0
                        %fprintf('Percent done = %d\n', round(100 * i / numel(headGrid.xCube)));
                        pr.progress(i / numel(headGrid.xCube), sprintf('\npercent done %d/100',round(100*i / numel(headGrid.xCube))));
                    end;
                    
                    pos = [headGrid.xCube(i) headGrid.yCube(i) headGrid.zCube(i)];
                    distanceToDipoles = sum((dipoleLocation - repmat(pos, size(dipoleLocation,1), 1))' .^2) .^ 0.5;
                    
                    % truncate the dipole denisty Gaussian at ~3 standard deviation
                    dipoleWithNonzeroWeightIds = distanceToDipoles < (standardDeviationOfEstimatedDipoleLocation * numberOfStandardDeviationsToTruncatedGaussaian);
                    
                    if ~isempty(dipoleWithNonzeroWeightIds)
                        
                        % pass distance to dipoles through a gaussian kernel with specified standard deviation.
                        gaussianPassedDistanceToDipoles = sqrt(1/(2 * pi * standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo)) * exp(-distanceToDipoles(dipoleWithNonzeroWeightIds).^2 / (2 * standardDeviationOfEstimatedErrorInDipoleLocationPowerTwo));
                        
                        if normalizeInBrainDipoleDenisty
                            % make dipole denisty sum inside brain volume equal to 1
                            gaussianPassedDistanceToDipoles = gaussianPassedDistanceToDipoles .* dipoleInBrainDensityNormalizationFactor(dipoleWithNonzeroWeightIds)';
                        end;
                        
                        gaussianWeightMatrix = repmat(gaussianPassedDistanceToDipoles,length(gaussianPassedDistanceToDipoles),1);
                        
                        normalizationMatrix = gaussianWeightMatrix .* gaussianWeightMatrix';
                        sumPairwiseWeights = sum(normalizationMatrix(:));
                        
                        normalizationMatrix = normalizationMatrix / sumPairwiseWeights;
                                                
                        similarityWeightedByGauissian = similarity(dipoleWithNonzeroWeightIds, dipoleWithNonzeroWeightIds) .* normalizationMatrix;
                        sumWeightedSimilarity(i) = sum(similarityWeightedByGauissian(:));
                        
                        % bootstapping with permutation
                        for permutationNumber = 1:numberOfPermutations
                            surrogateSimilarityWeightedByGauissian = similarity(randomPermutation(permutationNumber, dipoleWithNonzeroWeightIds), randomPermutation(permutationNumber, dipoleWithNonzeroWeightIds)) .* normalizationMatrix;
                            surrogateSumWeightedSimilarity(permutationNumber) = sum(surrogateSimilarityWeightedByGauissian(:));
                        end;
                        
                        sumWeightedSimilarityPvalue(i) = sum(surrogateSumWeightedSimilarity >= sumWeightedSimilarity(i)) / numberOfPermutations;
                    end;
                end;
            end;
            
            pause(.1);
            pr.progress('close'); % duo to some bug need a pause() before
            fprintf('\n');
            
            % we can only be sure that the significance is less than 1/numberOfPermutations
            sumWeightedSimilarityPvalue = max(sumWeightedSimilarityPvalue, (1 / numberOfPermutations) - eps);
        end
    end;
    methods
        function obj = meanProjection(dipoleAndMeasure, similarity, headGrid, varargin)
            % meanProjection(dipoleAndMeasure, similarity, headGrid, [optional key,values])
            % options:
            %
            % stdOfDipoleGaussian
            % numberOfPermutations
            % numberOfStdsToTruncateGaussian
            
            inputOptions = finputcheck(varargin, ...
                {'stdOfDipoleGaussian'         'real'  [] obj.standardDeviationOfEstimatedDipoleLocation; ...
                'numberOfPermutations'        'real'    [] obj.numberOfPermutations;...
                'numberOfStdsToTruncateGaussian'   'real'    []  obj.numberOfStandardDeviationsToTruncatedGaussaian  });
            
            if nargin < 2
                obj.headGrid = pr.headGrid;
            end;            
            obj.standardDeviationOfEstimatedDipoleLocation = inputOptions.stdOfDipoleGaussian;
            obj.numberOfPermutations = inputOptions.numberOfPermutations;
            obj.numberOfStandardDeviationsToTruncatedGaussaian = inputOptions.numberOfStdsToTruncateGaussian;
            obj.headGrid = headGrid;
            obj.similarity = similarity;
            
            [obj.convergenceSignificance obj.convergence] = pr.meanProjection.calculateMeasureConvergenceSignificance(obj.similarity, dipoleAndMeasure, obj.headGrid, obj.standardDeviationOfEstimatedDipoleLocation, obj.numberOfPermutations, obj.numberOfStandardDeviationsToTruncatedGaussaian);            
        end;
        function plotScatter(obj, significanceLevel)
            
            if nargin<2
                significanceLevel = 0.002;
            end;
            
            figure;
            
            t = obj.convergence - min(obj.convergence(:)) + 0.001;
            t = t / max(t(:));
            t(~obj.headGrid.insideBrainCube) = eps;
            
            t(obj.convergenceSignificance > significanceLevel) = eps;
            
            color = value2color(t(:), jet);
            
            scatter3(obj.headGrid.xCube(obj.headGrid.insideBrainCube), obj.headGrid.yCube(obj.headGrid.insideBrainCube), obj.headGrid.zCube(obj.headGrid.insideBrainCube), (t(obj.headGrid.insideBrainCube) * 40)+5, color(obj.headGrid.insideBrainCube), 'filled');
            axis equal;
            cbar;
        end;
        function [valueOnFineGridOutput mri] = plotMri(obj, significanceLevel, mri3dplotOptions)
            
            if nargin<2
                significanceLevel = 0.002;
            end;
            
            if nargin<3
                mri3dplotOptions = {'mriview' , 'top','mrislices', [-50 -30 -20 -15 -10 -5 0 5 10 15 20 25 30 40 50]};
            end;
            
            valueOnCoarseGrid = obj.convergence;
            valueOnCoarseGrid(obj.convergenceSignificance > significanceLevel) = 0;
            
            valueOnCoarseGrid(valueOnCoarseGrid<0) = 0;
            
            [valueOnFineGrid mri] = convert_coarse_grid_to_mri3d(valueOnCoarseGrid, obj.headGrid.xCube, obj.headGrid.yCube, obj.headGrid.zCube);
            mri3dplot(valueOnFineGrid, mri, mri3dplotOptions{:}); % for some reason, this function hicjacks a currently open figure. even if a new figur is just created.
            
            if nargout > 0
                valueOnFineGridOutput = valueOnFineGrid;
            end;
        end;
        function [obj position]= createDomain(obj, dipoleAndMeasure, similarityThreshold, significanceLevel, minNumberOfClusters, outlierThreshold)
            % [obj position]= createDomain(obj, dipoleAndMeasure, similarityThreshold, significanceLevel)
            
            if nargin<3
                similarityThreshold = 0.8;
            end;
            
            if nargin<4
                significanceLevel = 0.002;
            end;
            
            if nargin<5
                minNumberOfClusters = 2;
            end;
            if nargin<5
                outlierThreshold = 0.75;
            end;
            
            obj.significanceLevelForDomain = significanceLevel;
            
            significantId = obj.convergenceSignificance < significanceLevel;
            
            if any(significantId(:))
                
                %position = [obj.headGrid.xCube(significantId) obj.headGrid.yCube(significantId) obj.headGrid.zCube(significantId)];
                %[obj.linearizedProjectedMeasure obj.dipoleDensity] = dipoleAndMeasure.projectTo(position, obj.standardDeviationOfEstimatedDipoleLocation);
                                
                [projectionMatrix obj.dipoleDensity]= obj.getProjectionMatrix(dipoleAndMeasure, obj.headGrid, obj.standardDeviationOfEstimatedDipoleLocation, obj.numberOfStandardDeviationsToTruncatedGaussaian, significantId);
                obj.linearizedProjectedMeasure = dipoleAndMeasure.linearizedMeasure * projectionMatrix;
                
                projectedMeasureSimilarity = 1-squareform(pdist(obj.linearizedProjectedMeasure','correlation'));
                clusterInfo = cluster_based_on_similarities(projectedMeasureSimilarity, outlierThreshold, similarityThreshold, minNumberOfClusters, '3d');
                
                obj.domainNumberCube = obj.headGrid.xCube * 0;
                obj.domainNumberCube(significantId) = clusterInfo.IDX;
                
                % empty the most memory-consuming part of dipoleAndMeasure and retain a copy on it in
                % the projection object
                dipoleAndMeasure.linearizedMeasure = [];
                
                obj.domain = pr.domain;
                domainLabel = {};
                
                if clusterInfo.numberOfOutliers > 0
                    numberOfNonOutlierClusters = clusterInfo.numberOfClusters - 1;
                else
                    numberOfNonOutlierClusters = clusterInfo.numberOfClusters;
                end;
                
                for domainNumber = 1:numberOfNonOutlierClusters % one cluster is for the outliers
                    significantIndiceNumbers = find(significantId);
                    exemplarIdInHeadGrid = significantIndiceNumbers(clusterInfo.examplarId(domainNumber));
                    obj.domain(domainNumber) = pr.domain(obj.domainNumberCube == domainNumber, obj.headGrid, obj.linearizedProjectedMeasure(:, clusterInfo.IDX == domainNumber), exemplarIdInHeadGrid, obj.linearizedProjectedMeasure(:,clusterInfo.examplarId(domainNumber)), ['Domain ' num2str(domainNumber)], dipoleAndMeasure, obj.standardDeviationOfEstimatedDipoleLocation, obj.numberOfStandardDeviationsToTruncatedGaussaian);
                    domainLabel{domainNumber} = ['Domain ' num2str(domainNumber)];
                end;
                
                % place legend labels on the plot created by cluster_based_on_similarities()
                % add an Outlier label if needed.
                if clusterInfo.numberOfOutliers > 0
                    legend(['Outlier' domainLabel]);
                else
                    legend(domainLabel);
                end;
            else
                fprintf('There is no location with specified significance to create domain from. \nYou may choose a less strict significance criteria (higher p value) and try again.\n');
            end;
        end;
        function plotDomainScatter(obj, domainNumber)
            
            if nargin<2
                domainNumber = 1:length(obj.domain);
            end;
            
            figure;
            domainId = ismember(obj.domainNumberCube, domainNumber);
            pointSize = ones(size(obj.headGrid.xCube));
            pointSize(domainId) = 50;
            
            valueforColor = pointSize * 0;
            valueforColor(domainId) = obj.domainNumberCube(domainId);
            
            color = value2color(valueforColor(:), hsv(length(domainNumber)+1));
            
            % set the color of points that belong to no domains as black
            color(valueforColor(:) == 0,:) = 0;
            
            scatter3(obj.headGrid.xCube(obj.headGrid.insideBrainCube), obj.headGrid.yCube(obj.headGrid.insideBrainCube), obj.headGrid.zCube(obj.headGrid.insideBrainCube), pointSize(obj.headGrid.insideBrainCube) +5, color(obj.headGrid.insideBrainCube,:), 'filled');
            axis equal;
        end;
        function difference = findGroupDifference(obj, groupNameOrId, dipoleAndMeasure, similarity, headGrid, standardDeviationOfEstimatedDipoleLocation, numberOfPermutations)
            % groupId can be only either one(1) or two(2)
            
            groupId1 = ismemer(dipoleAndMeasure.groupName, groupNameOrId{1});
            groupId2 = ismemer(dipoleAndMeasure.groupName, groupNameOrId{2});
            
            groupId = groupId1 + 2 * groupId2;
            
            if nargin<7
                numberOfPermutations = 300;
            end;
            
            dipoleLocation = dipoleAndMeasure.location;
            
            standardDeviationOfEstimatedDipoleLocationPowerTwo = standardDeviationOfEstimatedDipoleLocation ^ 2;
            
            group1SignificanceInNeighborhood = ones(headGrid.cubeSize);
            group2SignificanceInNeighborhood = ones(headGrid.cubeSize);
            minGroupSignificanceInNeighborhood = ones(headGrid.cubeSize);
            
            groupDensityDifferenceSignificance =  ones(headGrid.cubeSize);
            normalizedGroup1Density = ones(headGrid.cubeSize);
            normalizedGroup2Density = ones(headGrid.cubeSize);
            
            surrogateGroupIds = create_surrogate_group_ids(groupId, numberOfPermutations, subjectId);
            
            for i=1:numel(gridX)
                if insideBrain(i) %% & sumWeightedSimilarityPvalue(i) < 0.02
                    if mod(i,10) ==0
                        fprintf('Percent done = %d\n', round(100 * i / numel(gridX)));
                    end;
                    
                    pos = [gridX(i) gridY(i) gridZ(i)];
                    distanceToDipoles = sum((dipoleLocation - repmat(pos, size(dipoleLocation,1), 1))' .^2) .^ 0.5;
                    
                    % pass distance to dipoles through a gaussian kernel with specified standard deviation.
                    gaussianPassedDistanceToDipoles = sqrt(1/(2 * pi * standardDeviationOfEstimatedDipoleLocationPowerTwo)) * exp(-distanceToDipoles.^2 / (2 * standardDeviationOfEstimatedDipoleLocationPowerTwo));
                    
                    gaussianWeightMatrix = repmat(gaussianPassedDistanceToDipoles,length(gaussianPassedDistanceToDipoles),1);
                    
                    % a matrix that weights each pair of dipoles according to their gaussians
                    neighborhoodWeightMatrix = gaussianWeightMatrix .* gaussianWeightMatrix';
                    
                    groupSignificanceInNeighborhood = calculate_group_significance_based_on_pairwise_similarity(groupId, similarity, surrogateGroupIds, neighborhoodWeightMatrix);
                    group1SignificanceInNeighborhood(i) = groupSignificanceInNeighborhood(1);
                    group2SignificanceInNeighborhood(i) = groupSignificanceInNeighborhood(2);
                    minGroupSignificanceInNeighborhood(i) = min(groupSignificanceInNeighborhood);
                    
                    [groupDensityDifferenceSignificance(i) normalizedGroup1Densities]= calculate_group_density_difference_significance(groupId, gaussianPassedDistanceToDipoles, surrogateGroupIds);
                    normalizedGroup1Density(i) = normalizedGroup1Densities(1);
                    normalizedGroup2Density(i) = normalizedGroup1Densities(2);
                end;
            end;
            
            
            difference.numberOfPermutations = numberOfPermutations;
            difference.group1SignificanceInNeighborhood = group1SignificanceInNeighborhood;
            difference.group2SignificanceInNeighborhood = group2SignificanceInNeighborhood;
            difference.minGroupSignificanceInNeighborhood = minGroupSignificanceInNeighborhood;
            difference.groupDensityDifferenceSignificance = groupDensityDifferenceSignificance;
            difference.normalizedGroup1Density = normalizedGroup1Density;
            difference.normalizedGroup2Density = normalizedGroup2Density;
            
        end
        function plotVolume(obj, significanceLevel)
            % plots the volume with significant convergence.
            % if domains are calculated, it will plot them with different colors on the same plot
            
            if nargin<2
                if isempty(obj.significanceLevelForDomain)
                    significanceLevel = 0.002;
                else
                    significanceLevel = obj.significanceLevelForDomain;
                end;
            end;
            
            figure;
            plot_dipplot_with_cortex;
            
            % if domain are not calculated, plot the significant volume.
            if isempty(obj.domainNumberCube)
                membershipCube = obj.convergenceSignificance <= significanceLevel;
                
                plot_head_region(obj.headGrid, membershipCube, 'g');
            else
                domainNumberCubeForPlot = obj.domainNumberCube;
                
                % domain points that are less significant than given significance value
                domainNumberCubeForPlot(obj.convergenceSignificance > significanceLevel) = 0;
                
                % choose domain colors based on their similarity
                for i=1:length(obj.domain)
                    domainExemplarMeasure(i,:) = obj.domain(i).exemplarLinearMeasure';
                end;
                
                if length(obj.domain) > 1
                    pairwiseSimilarityOfDomainExemplarMeasure = corrcoef(domainExemplarMeasure');
                    pairwiseDissimilarityOfDomainExemplarMeasure = 1- double(pairwiseSimilarityOfDomainExemplarMeasure);
                    pairwiseDissimilarityOfDomainExemplarMeasure = pairwiseDissimilarityOfDomainExemplarMeasure - diag(diag(pairwiseDissimilarityOfDomainExemplarMeasure));
                    
                    for i=1:size(pairwiseDissimilarityOfDomainExemplarMeasure,1)
                        for j=1:i
                            pairwiseDissimilarityOfDomainExemplarMeasure(i,j) = pairwiseDissimilarityOfDomainExemplarMeasure(j,i);
                        end;
                    end;
                    
                    try
                        pos = mdscale(pairwiseDissimilarityOfDomainExemplarMeasure, 1);
                    catch
                        pos = mdscale(pairwiseDissimilarityOfDomainExemplarMeasure, 1,'start' , 'random');
                    end;
                    
                    hsvFromRedToBlue = hsv;
                    hsvFromRedToBlue = hsvFromRedToBlue(1:44,:);
                    
                    domainColor = value2color(pos, hsvFromRedToBlue);
                else
                    domainColor = [0 1 0];
                end;
                
                for i=1:length(obj.domain)
                    plot_head_region(obj.headGrid, domainNumberCubeForPlot == i, domainColor(i,:));
                end;
                
            end;
        end;
        function plotVolumeColoredByMeasure(obj, dipoleAndMeasure, significanceLevel, varargin)
            % plotVolumeColoredByMeasure(dipoleAndMeasure, significanceLevel, 'key', 'value'..)
            
            inputOptions = finputcheck(varargin, ...
                {'colormap'         'string'  {'hsv' 'ycrcb'} 'hsv'; ...
                'whiteness'                 'real'    [0 1] 0.45;...
                });
            
            if nargin < 2
                error('Please provide a variable of type dipoleAndMeasure in the first argument.');
            end;
            
            if nargin<3
                if isempty(obj.significanceLevelForDomain)
                    significanceLevel = 0.002;
                else
                    significanceLevel = obj.significanceLevelForDomain;
                end;
            end;
            
            significantId = obj.convergenceSignificance < significanceLevel;
            [projectionMatrix totalDipoleDenisty]= obj.getProjectionMatrix(dipoleAndMeasure, obj.headGrid, obj.standardDeviationOfEstimatedDipoleLocation, obj.numberOfStandardDeviationsToTruncatedGaussaian, significantId);
            linearizedProjectedMeasure = dipoleAndMeasure.linearizedMeasure * projectionMatrix;
            
            % create a 1-D multi-dimensional scaling (MDS) from projected similarities.
            projectedMeasureSimilarity = squareform(pdist(linearizedProjectedMeasure','correlation'));
            
            % colorbased on a multi-dimensional scaling of the projected measure
            if isempty(projectedMeasureSimilarity)
                fprintf('There are not significant portions to be plotted. Consider increasing the significance value.\n');
            else
                if strcmpi(inputOptions.colormap, 'ycrcb')
                    % use YCrCb color mapping
                    [pos stress]= mdscale(projectedMeasureSimilarity, 2);
                    
                    pos = pos /  max(range(pos));
                    pos(:,1) = pos(:,1) - min(pos(:,1));
                    pos(:,2) = pos(:,2) - min(pos(:,2));
                    Y = ones(size(pos,1), 1) * inputOptions.whiteness; % Y value in ycbcr, related to luminescence (between 0 for dark and 1 for all white)
                    
                    voxelColor = ycbcr2rgb([Y pos]);
                else % hsv color mapping
                    [pos stress]= mdscale(projectedMeasureSimilarity, 1);
                    
                    % map the 1-D representation of projected measures into color
                    hsvFromRedToBlue = hsv;
                    hsvFromRedToBlue = hsvFromRedToBlue(1:44,:);
                    
                    voxelColor = value2color(pos, hsvFromRedToBlue);
                end;
               
                fprintf('Multi-Dimensional Scaling stress (deformation) for representing projected measures by color = %%%d\n', round(100 * stress));

                figure;
                plot_dipplot_with_cortex;
                
                idNumbers = find(significantId);
                for i=1:length(pos)
                    membershipCube = logical(zeros(obj.headGrid.cubeSize));
                    membershipCube(idNumbers(i)) = 1;
                    plot_head_region(obj.headGrid, membershipCube, voxelColor(i,:));
                end;
            end;
        end;
        function plotVolumeColoredByDomain(obj, varargin)
            % plotVolumeColoredByMeasure(dipoleAndMeasure, significanceLevel, 'key', 'value'..)
            
            inputOptions = finputcheck(varargin, ...
                {'colormap'         'string'  {'hsv' 'ycrcb'} 'hsv'; ...
                'whiteness'                 'real'    [0 1] 0.45;...
                });
            
            if isempty(obj.domain) 
                error('No domains are present. Please first create domain(s) with createDomain() method.\n');                
            end;
                   
            % if only one domain exist, just plot it (no need for different colors with mds()...)
            if numel(obj.domain) == 1
                obj.domain(1).plotVolume;
                return;
            end;
            
            for i=1:length(obj.domain)
                linearizedProjectedMeasure(i,:) =  obj.domain(i).exemplarLinearMeasure;
            end;
            
            % create a 1-D multi-dimensional scaling (MDS) from projected similarities.
            projectedMeasureSimilarity = squareform(pdist(linearizedProjectedMeasure,'correlation'));
            
            % colorbased on a multi-dimensional scaling of domain exemplars
            
            if strcmpi(inputOptions.colormap, 'ycrcb')
                % use YCrCb color mapping
                [pos stress]= mdscale(projectedMeasureSimilarity, 2);
                
                pos = pos /  max(range(pos));
                pos(:,1) = pos(:,1) - min(pos(:,1));
                pos(:,2) = pos(:,2) - min(pos(:,2));
                Y = ones(size(pos,1), 1) * inputOptions.whiteness; % Y value in ycbcr, related to luminescence (between 0 for dark and 1 for all white)
                
                voxelColor = ycbcr2rgb([Y pos]);
            else % hsv color mapping
                [pos stress]= mdscale(projectedMeasureSimilarity, 1);
                
                % map the 1-D representation of projected measures into color
                hsvFromRedToBlue = hsv;
                hsvFromRedToBlue = hsvFromRedToBlue(1:44,:);
                
                voxelColor = value2color(pos, hsvFromRedToBlue);
            end;
            
            fprintf('Multi-Dimensional Scaling stress (deformation) for representing domain projected measure exemplars by color = %%%d\n', round(100 * stress));
            
            figure;
            plot_dipplot_with_cortex;
            
            % remove all legends (set them to 'off')
            pr.remove_all_legends_from_figure;
            
            for i=1:length(obj.domain)
                domainHgGroup(i) = hggroup('DisplayName', ['Domain ' num2str(i)]);
                set(get(get(domainHgGroup(i), 'annotation'), 'LegendInformation'),'IconDisplayStyle', 'on');
                plot_head_region(obj.headGrid, obj.domain(i).membershipCube, voxelColor(i,:), {'parent', domainHgGroup(i)});                                                
                plot_head_region_projected_on_mrs(obj.headGrid, obj.domain(i).membershipCube, voxelColor(i,:), {'parent', domainHgGroup(i)});                
            end;
            
            
            legendHandle = legend('show');
            set(legendHandle,  'textcolor', [1 1 1]);
        end;
    end
end